﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using Windows.Devices.Enumeration;
using Windows.Devices.Spi;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;

// Die Vorlage "Leere Seite" ist unter http://go.microsoft.com/fwlink/?LinkId=402352&clcid=0x409 dokumentiert.

namespace Bus1
{
    /// <summary>
    /// Eine leere Seite, die eigenständig verwendet oder zu der innerhalb eines Rahmens navigiert werden kann.
    /// </summary>
    public sealed partial class MainPage : Page
    {
        SpiDevice spiDevice;

        public MainPage()
        {
            this.InitializeComponent();
            Loaded += MainPage_Loaded;
            Unloaded += MainPage_Unloaded;
        }

        private async void MainPage_Loaded(object sender, Windows.UI.Xaml.RoutedEventArgs e)
        {
            // SPI ermitteln
            string selector = SpiDevice.GetDeviceSelector();
            IReadOnlyList<DeviceInformation> devices = await DeviceInformation.FindAllAsync(selector);

            // SPI Einstellungen setzen
            var settings = new SpiConnectionSettings(0)
            {
                ClockFrequency = 5000000,
                Mode = SpiMode.Mode3,
            };
            spiDevice = await SpiDevice.FromIdAsync(devices[0].Id, settings);
            if (spiDevice == null)
            {
                Debug.WriteLine("SPI0 in use");
                Application.Current.Exit();
            }
            details.Text += GetSpiDetails(devices);

            // Initialisierungskommandos 
            if (!IsADXL345())
            {
                Debug.WriteLine("No ADXL345 accelerometer found");
                Application.Current.Exit();
            }

            // Datenformat setzen (Genauigkeit: +/-8G, siehe Datenblatt, S. 17)
            // DATA_FORMAT register: 0x31(A0 - A7), 0x02 = 0b00000010(D0 - D7)
            spiDevice.Write(new byte[] { 0x31, 0x02 });

            // POWER_CTL register: 0x2D; 0x08 = 0b00001000 = Messmodus einschalten (ansonsten Standby) (Data sheet: S. 16)
            spiDevice.Write(new byte[] { 0x2D, 0x08 });

            // Timer zum Abruf der Daten, kürzester Intervall: 0 übergeben
            var timer = new DispatcherTimer() { Interval = TimeSpan.FromMilliseconds(50) };
            // DispatcherTimer läuft gegenüber Timer und ThreadPoolTimer im UI-Thread
            timer.Tick += Timer_Tick;
            timer.Start();
        }

        private string GetSpiDetails(IReadOnlyList<DeviceInformation> devices)
        {
            string details = "SpiDevices\n\n";

            foreach (var device in devices)
            {
                SpiBusInfo busInfo = SpiDevice.GetBusInfo(device.Id);

                details += $"Device: {device.Id}\n";
                details += $"ChipSelectCount: {busInfo.ChipSelectLineCount}\n";
                details += $"MinClockFrequency: {busInfo.MinClockFrequency}\n";
                details += $"MaxClockFrequency: {busInfo.MaxClockFrequency}\n";
                details += "SupportedDataBitLengths: ";
                foreach (var dataBitLength in busInfo.SupportedDataBitLengths)
                {
                    details += dataBitLength + " ";
                }
                details += "\n";
            }
            return details;
        }

        private bool IsADXL345()
        {
            // DEVICE_ID register: 0x00
            var readBuffer = new byte[2];

            spiDevice.Write(new byte[] { 0x00 | 0x80, 0x00 });
            spiDevice.Read(readBuffer);

            if (readBuffer[0] != 0xe5)  // feste DEVICE_ID für ADXL345
            {
                Debug.WriteLine($"{readBuffer[0]:x}");
                return false;
            }

            return true;
        }

        private void Timer_Tick(object sender, object e)
        {
            // Gleiche Arraygröße bei Aufruf von TransferFullDuplex   
            byte[] writeBuffer = new byte[1 + 6];
            byte[] readBuffer = new byte[1 + 6];

            // Mehrere Datenregister lesen
            // DATAX0, DATAX1, DATAY0, DATAY1, DATAZ0, DATAZ1 register: 0x32; Bit 6: 0x40 = Multiple-Byte-Read/Write, Bit 8: 0x80 = Read/Write (Datenblatt: Figure 5/6)
            writeBuffer[0] = 0x32 | 0x80 | 0x40;
            spiDevice.TransferFullDuplex(writeBuffer, readBuffer);

            // zwei 8-Bit-Werte zur Int16 zusammensetzen, erstes Byte mit Command DATAX0 nicht berücksichtigen
            var x = ConvertToG(BitConverter.ToInt16(readBuffer, 1));
            var y = ConvertToG(BitConverter.ToInt16(readBuffer, 3));
            var z = ConvertToG(BitConverter.ToInt16(readBuffer, 5));

            UpdatePage(x, y, z);
        }

        private double ConvertToG(Int16 rawValue)
        {
            const int RESOLUTION = 1024;          // 10 Bit-Auflösung = 2 hoch 10 = 1024
            const int RANGE = 8 + 8;              // +/-8g
            double unitsPerG = RESOLUTION / RANGE;

            return rawValue / unitsPerG;
        }

        private void UpdatePage(double x, double y, double z)
        {
            XAxis.Text = $"X-Achse: {x:F3}G";
            YAxis.Text = $"Y-Achse: {y:F3}G";
            ZAxis.Text = $"Z-Achse: {z:F3}G";
        }

        private void MainPage_Unloaded(object sender, Windows.UI.Xaml.RoutedEventArgs e)
        {
            spiDevice.Dispose();
        }

    }
}
