﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Threading.Tasks;
using Windows.Devices.Enumeration;
using Windows.Devices.I2c;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;

// Die Vorlage "Leere Seite" ist unter http://go.microsoft.com/fwlink/?LinkId=402352&clcid=0x409 dokumentiert.

namespace Bus2
{
    /// <summary>
    /// Eine leere Seite, die eigenständig verwendet oder zu der innerhalb eines Rahmens navigiert werden kann.
    /// </summary>
    public sealed partial class MainPage : Page
    {
        I2cDevice i2cDevice;

        public MainPage()
        {
            this.InitializeComponent();

            Loaded += MainPage_Loaded;
            Unloaded += MainPage_Unloaded;
        }

        private async void MainPage_Loaded(object sender, Windows.UI.Xaml.RoutedEventArgs e)
        {
            // I2C ermitteln
            string selector = I2cDevice.GetDeviceSelector();
            IReadOnlyList<DeviceInformation> devices = await DeviceInformation.FindAllAsync(selector);

            // I2C Einstellungen setzen: 0x1D = SDO an 3,3V, 0x53 = SDO an GND
            var settings = new I2cConnectionSettings(0x1D)
            {
                BusSpeed = I2cBusSpeed.FastMode
            };
            i2cDevice = await I2cDevice.FromIdAsync(devices[0].Id, settings);
            if (i2cDevice == null)
            {
                Debug.WriteLine("I2C1 in use");
                Application.Current.Exit();
            }

            details.Text += GetI2CDetails(devices);

            // Initialisierungskommandos 
            if (!IsADXL345())
            {
                Debug.WriteLine("No ADXL345 accelerometer found");
                Application.Current.Exit();
            }

            // Datenformat setzen (Genauigkeit: +/-8G, siehe Datenblatt, S. 17)
            // DATA_FORMAT register: 0x31(A0 - A7), 0x02 = 0b00000010(D0 - D7)
            i2cDevice.Write(new byte[] { 0x31, 0x02 });

            // POWER_CTL register: 0x2D; 0x08 = 0b00001000 = Messmodus einschalten (ansonsten Standby) (Data sheet: S. 16)
            i2cDevice.Write(new byte[] { 0x2D, 0x08 });

            // Timer zum Abruf der Daten, kürzester Intervall: 0 übergeben
            var timer = new DispatcherTimer() { Interval = TimeSpan.FromMilliseconds(50) };
            timer.Tick += Timer_Tick;
            timer.Start();
        }

        private string GetI2CDetails(IReadOnlyList<DeviceInformation> devices)
        {
            string details = "I2CDevices\n\n";

            foreach (var device in devices)
            {
                details += $"Device: {device.Id}\n";
                details += $"Name: {device.Name}\n";
                foreach (var property in device.Properties)
                {
                    details += $"{property.Key}: {property.Value}\n";
                }
            }
            return details;
        }

        private bool IsADXL345()
        {
            // DEVICE_ID register: 0x00
            var readBuffer = new byte[2];

            i2cDevice.WriteRead(new byte[] { 0x00 }, readBuffer);

            if (readBuffer[0] != 0xe5)  // feste DEVICE_ID für ADXL345
            {
                Debug.WriteLine("No ADXL345 accelerometer found");
                return false;
            }

            return true;
        }

        private void Timer_Tick(object sender, object e)
        {
            byte[] readBuffer = new byte[6];

            // Mehrere Datenregister lesen
            // DATAX0, DATAX1, DATAY0, DATAY1, DATAZ0, DATAZ1 register: 0x32
            i2cDevice.WriteRead(new byte[] { 0x32 }, readBuffer);

            // zwei 8-Bit-Werte zu Int16 zusammensetzen
            var x = ConvertToG(BitConverter.ToInt16(readBuffer, 0));
            var y = ConvertToG(BitConverter.ToInt16(readBuffer, 2));
            var z = ConvertToG(BitConverter.ToInt16(readBuffer, 4));

            UpdatePage(x, y, z);
        }

        private double ConvertToG(Int16 rawValue)
        {
            const int RESOLUTION = 1024;          // 10 Bit-Auflösung = 2 hoch 10 = 1024
            const int RANGE = 8 + 8;              // +/-8g
            double unitsPerG = RESOLUTION / RANGE;

            return rawValue / unitsPerG;
        }

        private async void UpdatePage(double x, double y, double z)
        {
            await Dispatcher.RunIdleAsync((e) =>
            {
                XAxis.Text = $"X-Achse: {x:F3}G";
                YAxis.Text = $"Y-Achse: {y:F3}G";
                ZAxis.Text = $"Z-Achse: {z:F3}G";
            });
        }

        private void MainPage_Unloaded(object sender, Windows.UI.Xaml.RoutedEventArgs e)
        {
            i2cDevice.Dispose();
        }
    }
}
