﻿using System;
using System.Collections.Generic;
using System.Threading.Tasks;
using Windows.Devices.Enumeration;
using Windows.Devices.I2c;
using Windows.UI.Xaml;

namespace Bus3
{
    public class SensorEventArgs
    {
        public SensorEventArgs(double value)
        {
            Value = value;
        }
        public double Value { get; private set; }
    }

    public class BH1750Sensor : IDisposable
    {
        I2cDevice i2cDevice;
        byte slaveAdress;
        DispatcherTimer timer;
        TimeSpan interval = TimeSpan.FromMilliseconds(180);

        public TimeSpan Interval
        {
            get
            {
                return interval;
            }
            set
            {
                timer.Interval = value;
            }
        }

        public delegate void SensorEventHandler(object sender, SensorEventArgs e);
        public event SensorEventHandler ValueChanged;

        private BH1750Sensor() { }

        public async static Task<BH1750Sensor> Create(byte slaveAddress = 0x23)
        {
            var sensor = new BH1750Sensor();
            sensor.slaveAdress = slaveAddress;

            // I2C ermitteln
            string selector = I2cDevice.GetDeviceSelector();
            IReadOnlyList<DeviceInformation> devices = await DeviceInformation.FindAllAsync(selector);
            if (devices.Count == 0)
            {
                throw new ArgumentException("No I2C bus found");
            }

            // I2C Einstellungen setzen: 0x23
            var settings = new I2cConnectionSettings(slaveAddress)
            {
                BusSpeed = I2cBusSpeed.FastMode
            };
            sensor.i2cDevice = await I2cDevice.FromIdAsync(devices[0].Id, settings);
            if (sensor.i2cDevice == null)
            {
                throw new ArgumentException("I2C1 in use");
            }

            // Initialisierungskommandos 
            if (!sensor.IsBH1750())
            {
                throw new ArgumentException("No BH1750 sensor found");
            }

            sensor.timer = new DispatcherTimer() { Interval = sensor.interval };
            sensor.timer.Tick += sensor.Timer_Tick;
            sensor.timer.Start();

            return sensor;
        }

        private bool IsBH1750()
        {
            try
            {
                i2cDevice.Write(new byte[] { 0x00 });
            }
            catch
            {
                return false;
            }
            return true;
        }

        private void Timer_Tick(object sender, object e)
        {
            var readBuffer = new byte[2];
            i2cDevice.WriteRead(new byte[] { slaveAdress }, readBuffer);

            //Array.Reverse(readBuffer, 0, 2);
            //var value = BitConverter.ToInt16(readBuffer, 0) / 1.2;
            // oder
            var value = ((readBuffer[0] << 8) | readBuffer[1]) / 1.2;

            ValueChanged?.Invoke(this, new SensorEventArgs(value));
        }

        public void Dispose()
        {
            i2cDevice.Dispose();
        }

    }
}
