﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using Windows.Devices.Enumeration;
using Windows.Devices.Spi;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Shapes;

namespace Analog1
{
    public sealed partial class MainPage : Page
    {
        SpiDevice spiDevice;

        public MainPage()
        {
            this.InitializeComponent();
            Loaded += MainPage_Loaded;
            Unloaded += MainPage_Unloaded;
        }

        private async void MainPage_Loaded(object sender, RoutedEventArgs e)
        {
            // SPI ermitteln
            string selector = SpiDevice.GetDeviceSelector();
            IReadOnlyList<DeviceInformation> devices = await DeviceInformation.FindAllAsync(selector);

            // SPI Einstellungen setzen
            var settings = new SpiConnectionSettings(0)
            {
                ClockFrequency = 1200000,   // 1,2MHz bei 2,7V (siehe Datenblatt, S. 3)
                Mode = SpiMode.Mode0,
            };
            spiDevice = await SpiDevice.FromIdAsync(devices[0].Id, settings);
            if (spiDevice == null)
            {
                Debug.WriteLine("SPI0 in use");
                Application.Current.Exit();
            }

            // Timer zum Abruf der Daten, kürzester Intervall: 0 übergeben
            var timer = new DispatcherTimer() { Interval = TimeSpan.FromMilliseconds(0) };
            timer.Tick += Timer_Tick;
            timer.Start();
        }

        int xTemp = 0, yTemp = 0;
        int jitterThreshold = 20;

        private void Timer_Tick(object sender, object e)
        {
            int x = ReadValue(0);
            int y = ReadValue(1);

            if (Math.Abs(x - xTemp) >= jitterThreshold || Math.Abs(y - yTemp) >= jitterThreshold)
            {
                xTemp = x;
                yTemp = y;

                SetPosition(x, y, ellipse);
            }
            xCoord.Text = x.ToString();
            yCoord.Text = y.ToString();
        }

        private void SetPosition(double x, double y, Shape element)
        {
            var cx = canvas.ActualWidth - element.Width;
            var cy = canvas.ActualHeight - element.Height;

            x = (x / 1023 * cx);
            y = (y / 1023 * cy);

            Canvas.SetLeft(element, x);
            Canvas.SetTop(element, y);
        }

        private int ReadValue(byte channelNumber)
        {
            byte[] readBuffer = new byte[2];
            byte[] writeBuffer = new byte[2];

            // 0x40: Startbit (0b0100_0000), 7.tes Bit
            // 0x20: Single-Ended (0b0010_0000), 6.tes Bit
            // 0x10: Channel = 1 (0b0001_0000) 5.tes Bit
            // 0x08: Most-Significant-Bit (MSB)(0b0000_1000), 4.tes Bit
            writeBuffer[0] = (byte)(0x40 | 0x20);

            if (channelNumber == 1)
                writeBuffer[0] |= 0x10;

            spiDevice.TransferFullDuplex(writeBuffer, readBuffer);

            var value = (readBuffer[0] & 0x03) << 8;    // Byte 0: nur 2 Bit berücksichtigen
            value |= readBuffer[1];                     // Byte 1: 8 Bit 

            return value;
        }

        private void MainPage_Unloaded(object sender, RoutedEventArgs e)
        {
            spiDevice.Dispose();
        }
    }
}
