﻿using System;
using System.Collections.Generic;
using Windows.Devices.Adc.Provider;
using Windows.Devices.Enumeration;
using Windows.Devices.Spi;

namespace DataAppTemplate
{
    class Mcp3002Provider : IAdcProvider
    {
        public IReadOnlyList<IAdcControllerProvider> GetControllers()
        {
            List<IAdcControllerProvider> list = new List<IAdcControllerProvider>();
            list.Add(new Mcp3002Controller());

            return list;
        }
    }

    class Mcp3002Controller : IAdcControllerProvider
    {
        public int ChannelCount { get; } = 2;

        public int MaxValue { get; } = 1024 - 1;
        public int MinValue { get; } = 0;

        public int ResolutionInBits { get; } = 10;

        public ProviderAdcChannelMode ChannelMode { get; set; } = ProviderAdcChannelMode.SingleEnded;

        private SpiDevice spiDevice;

        public Mcp3002Controller()
        {
            Initialize();
        }

        private async void Initialize()
        {
            // SPI ermitteln
            string selector = SpiDevice.GetDeviceSelector();
            IReadOnlyList<DeviceInformation> devices = await DeviceInformation.FindAllAsync(selector);

            // SPI Einstellungen setzen
            var settings = new SpiConnectionSettings(0)
            {
                ClockFrequency = 1200000,   // 1,2MHz bei 2,7V (siehe Datenblatt, S. 3)
                Mode = SpiMode.Mode0
            };
            spiDevice = await SpiDevice.FromIdAsync(devices[0].Id, settings);
            if (spiDevice == null)
                throw new ArgumentException("SPI0 in use");
        }


        public void AcquireChannel(int channel)
        {
            if (channel < 0 || channel > ChannelCount)
                throw new ArgumentException($"Only channels between 0 and {ChannelCount} supported");
        }


        public void ReleaseChannel(int channel)
        {
            if (channel < 0 || channel > ChannelCount)
                throw new ArgumentException($"Only channels between 0 and {ChannelCount} supported");
        }

        public bool IsChannelModeSupported(ProviderAdcChannelMode channelMode)
        {
            return true;
        }

        public int ReadValue(int channelNumber)
        {
            byte[] readBuffer = new byte[6];
            byte[] writeBuffer = new byte[6];

            // 0x40: Startbit (0b0100_0000), 7.tes Bit
            // 0x20: single-ended (0b0010_0000), 6.tes Bit
            // 0x10: Channel = 1 (0b0001_0000) 5.tes Bit
            // 0x08: Most-Significant-Byte-Bit (MSB)(0b0000_1000), 4.tes Bit
            writeBuffer[0] = 0x40;
            if (ChannelMode == ProviderAdcChannelMode.SingleEnded)
                writeBuffer[0] |= 0x20;
            if (channelNumber == 1)
                writeBuffer[0] |= 0x10;
            writeBuffer[0] |= 0x08;

            spiDevice.TransferFullDuplex(writeBuffer, readBuffer);

            var value = (readBuffer[0] & 0x03) << 8;    // Byte 0: nur 2 Bit berücksichtigen
            value |= readBuffer[1];                     // Byte 1: 8 Bit 

            return value;

        }
    }
}
