﻿using Microsoft.Azure.Devices.Client;
using Newtonsoft.Json;
using System;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.Text;
using System.Threading.Tasks;
using Windows.Devices.Adc;
using Windows.Devices.Geolocation;
using Windows.Security.ExchangeActiveSyncProvisioning;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using WinRTXamlToolkit.Controls.DataVisualization.Charting;

namespace DataProducer
{
    public sealed partial class MainPage : Page
    {
        public MainPage()
        {
            this.InitializeComponent();
            Loaded += MainPage_Loaded;

            (LineChart.Series[0] as LineSeries).ItemsSource = new ObservableCollection<object>();
        }

        private async void MainPage_Loaded(object sender, RoutedEventArgs e)
        {
            string deviceId = new EasClientDeviceInformation().FriendlyName;
            await Register(deviceId);

            var geoloc = new Geolocator();
            var position = await geoloc.GetGeopositionAsync();
            var latitude = position.Coordinate.Point.Position.Latitude;
            var longitude = position.Coordinate.Point.Position.Longitude;

            var controller = (await AdcController.GetControllersAsync(new Mcp3002Provider()))[0];
            var channel = controller.OpenChannel(0);

            var timer = new DispatcherTimer()
            {
                Interval = new TimeSpan(0, 0, 0, 5)
            };

            timer.Tick += (s, args) =>
            {
                var rawValue = channel.ReadValue();

                var dataPoint = new Measurement()
                {
                    DeviceId = deviceId,
                    TimeOfEvent = DateTimeOffset.Now,
                    RawValue = rawValue,
                    Ratio = channel.ReadRatio(),
                    Value = ConvertToCelsius(rawValue),
                    Longitude = longitude,
                    Latitude = latitude
                };

                Debug.WriteLine(dataPoint);

                SendToCloudAsync(dataPoint);

                AddDataPoint(dataPoint.Value);
            };
            timer.Start();
        }

        private void AddDataPoint(double value)
        {
            var lineSeries = LineChart.Series[0] as LineSeries;
            var series = lineSeries.ItemsSource as ObservableCollection<object>;
            series.Add(new
            {
                TimeOfEvent = DateTime.Now,
                Value = value
            });

            // die letzten 50 Messungen anzeigen
            if (series.Count > 50)
                series.RemoveAt(0);
        }

        private double ConvertToCelsius(int value)
        {
            // Formel: http://www.linkerkit.de/index.php?title=KY-013_Temperatur-Sensor_Modul
            var temperature = Math.Log(10000.0 * ((1024.0 / value - 1)));
            temperature = 1 / (0.001129148 + (0.000234125 + (0.0000000876741 * temperature * temperature)) * temperature);
            temperature -= 273.15;    // von Kelvin in Celsius

            return temperature;
        }

        const string ConnectionIoTString = "HostName=<Ihr Host>;SharedAccessKeyName=iothubowner;SharedAccessKey=<Ihr Shared Key>";
        DeviceClient deviceClient;

        private async Task Register(string deviceId)
        {
            var registryManager = Microsoft.Azure.Devices.RegistryManager.CreateFromConnectionString(ConnectionIoTString);

            var device = await registryManager.GetDeviceAsync(deviceId);
            if (device == null)
            {
                device = new Microsoft.Azure.Devices.Device(deviceId);
                device = await registryManager.AddDeviceAsync(device);
            }
            if (device.Status == Microsoft.Azure.Devices.DeviceStatus.Enabled)
            {
                string connectionWithDeviceClient = $"DeviceId={deviceId};{ConnectionIoTString}";
                deviceClient = DeviceClient.CreateFromConnectionString(connectionWithDeviceClient); // TransportType.Amqp
            }
        }

        private async void SendToCloudAsync(Measurement measurement)
        {
            string data = JsonConvert.SerializeObject(measurement);
            Message message = new Message(Encoding.UTF8.GetBytes(data));

            await deviceClient.SendEventAsync(message);
        }



    }
}
